#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
    g2p.py
    ~~~~~~~~
    
    HTTP Proxy Server in Python.
    
    :copyright: (c) 2015 by Terrence Chin.
    :license: BSD, see LICENSE for more details.
"""

import sys
import datetime
import argparse
import re
import requests
import logging
import yaml

from actions.action import *
from BaseHTTPServer import HTTPServer, BaseHTTPRequestHandler
from urlparse import urlparse, parse_qs, urljoin

logger = logging.getLogger(__name__)


class Proxy(object):
    """
    This proxy make GET request be converted to POST request
    """

    def __init__(self, url, actions, headers):
        self.url = url
        self.actions = actions
        self.headers = headers

    def handle(self):
        request = {'url':self.url,
                   'headers':self.headers}
        for action in self.actions:
            request = action.handle(request)
        if request.get('error', None):
            return None, request['error']
        return requests.post(**request), None


class RequestHandler(BaseHTTPRequestHandler):
    
    def do_GET(self):
        
        request_path = self.path
        headers = self.headers
        parser = urlparse(request_path)
        params = parse_qs(parser.query)
        url = params.get('url', None)
        if url is not None:
            url = url[0] #['url'] -> 'url'
            self.post(url, params, headers)
        else:
            self.send_response(200)
            self.end_headers()

    def post(self, url, params, headers):
            logger.info("Forword request:%s" % url)
            proxy = Proxy(url, RequestHandler.actions, headers)
            result, error = proxy.handle()
            if error is not None:
                self.send_response(500)
                self.end_headers()
                self.wfile.write(error)
                self.finish() 
                return

            src = self.fetch_src(result.content, url, params.get('src', [None])[0])
            if src:
                self.post(src, params)
                return

            self.send_response(result.status_code)
            if result.headers.get('content-type', False):
                self.send_header('content-type', result.headers.get('content-type'))
            self.end_headers()
            self.wfile.write(result.content)
            self.finish() 

    def finish(self):
        if not self.wfile.closed:
            self.wfile.flush()
        self.wfile.close()

    def fetch_src(self, html, url, match_str):
        if match_str is None: return None

        src_list = re.findall("src=\"(.*?)\"", html)
        for src in src_list:
            if re.search(match_str, src):
                return urljoin(url, src)
        
    do_POST = do_GET    
    do_PUT = do_GET
    do_DELETE = do_GET


def main():
    parser = argparse.ArgumentParser(
        description='g2p.py --config xxx --port xxx',
        epilog=''
    )
    
    parser.add_argument('--port', default='8899', help='Default: 8899')
    parser.add_argument('--config', default='conf/action.yaml', help='ConfigPath: conf/action.yaml')
    parser.add_argument('--log-level', default='INFO', help='DEBUG, INFO, WARNING, ERROR, CRITICAL')
    args = parser.parse_args()
    
    logging.basicConfig(level=getattr(logging, args.log_level), format='%(asctime)s - %(levelname)s - pid:%(process)d - %(message)s')
    
    port = int(args.port)
    RequestHandler.actions = ActionFactory(args.config).build()
    
    try:
        logger.info("Start server on port=%s" % port)
        server = HTTPServer(('', port), RequestHandler)
        server.serve_forever()
    except KeyboardInterrupt:
        pass


if __name__ == '__main__':
    main()
